
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% QUDEN - Quantum-Transport Engineering of Devices for Nanoelectronics
% Version 2025 (Internal version base 8.0)
%
% Atomistic Quantum Transport Simulator for 2D Material Nanoribbons
%
% Authors: Assoc. Prof. Mirko Poljak, PhD 
%          Mislav Matić, PhD
%
% Contact email: mirko.poljak@fer.hr
%
% University of Zagreb, Faculty of Electrical Engineering and Computing, 
% Computational Nanoelectronics Group (www.fer.unizg.hr/conan)
%
% Copyright: (c) 2012-2024, All rights reserved.
% 
% Please cite: TBA
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;

% -------------------------------------------------------------------------
% Parallel execution, set no. of cores
% -------------------------------------------------------------------------

N_core = 8; % number of cores
if isempty(gcp('nocreate')); poolobj=parpool('Processes', N_core); end

% -------------------------------------------------------------------------
% Inputs - single gate FET parameters 
% -------------------------------------------------------------------------
%                         
%            _________________________________
%           |  S  |         G           |  D  |               
%           |_____|_____________________|_____|
%           |                                 |  ↑
%           |          oxide(epsil_r)         |  | tox
%           |_________________________________|  ↓
%           |     |                     |     |
%           |-----|-------channel-------|-----|
%           |_____|_____________________|_____|
%                  <------------------->
%                          Lx_c
%
% User needs to specify:
% 
% FET type, channel material, channel length (L_ch), oxide (thickness
% (tox) and dielectric material), S/D doping, 
% 
% NOTE: Source (S) and drain (D) are ideal, i.e. semi-infinite of the same material as the channel 
% -------------------------------------------------------------------------

version = 'v5'; % specify version of the simulation (if same version is used multiple times, it will overwrite the data)

% -------------------------------------------------------------------------
% single gate FET parameters needed for NEGF simulation
% -------------------------------------------------------------------------

FET_type = 'N'; %'N' for 'N'-type single gate FET, 'P' for p-type single gate 'FET'

material_name = "GNR"; % channel and S/D material. Name of the material obtained from material_selection.m
Nw_all = [3]; % nanoribbon widths for which simulations are made in terms of number of unit cells in
              % nanoribbon width -> for DFT-MLWF inputs see material_selection.m

Lx_c  = 15e-9; % channel length in m (length calculated by repeating unit cell along channel length, 
               % NOTE: actual might differ due to rounding error)

% -------------------------------------------------------------------------
% single gate FET parameters needed for TOB simulation
% -------------------------------------------------------------------------

tox = 1e-9; % oxide thickness (m)
epsil_r = 3.9; % relative permittivity (SiO2 3.9, Hf02 20)

frac = 0.01; % S/D doping defined as fraction of areal atom density (mat.NDmax defined in material_selection.m)

% -------------------------------------------------------------------------
% Select simulation (sim_num)
% -------------------------------------------------------------------------
% '1'  - NEGF and TOB
% '2'  - only NEGF simulation
% '3'  - only TOB simulation - CAUTION: TOB must be run after NEGF with the same NEGF parameters                                        
% -------------------------------------------------------------------------

sim_num = 1; %Select simulation 

% -------------------------------------------------------------------------
% Creating simulation environment
% -------------------------------------------------------------------------

addpath("HR_data",append("HR_data/",material_name),"TOOLS","TOOLS/NEGF","TOOLS/ToB")
d.home_folder = append(pwd,"/","OUTPUT/",material_name,"/",version,'/',FET_type,'FET/');
[status, msg, msgID] = mkdir(d.home_folder); %make dir for output

constants; %list of constants

% -------------------------------------------------------------------------
% Run selected simulations
% -------------------------------------------------------------------------

tic % simulation length - start counting

switch sim_num
    case 1
        main_NEGF
        main_tob 
    case 2
        main_NEGF
    case 3 
        main_tob
end

fprintf("\nSimulation time is %2.1f minutes\n",toc/60);
% simulation length - stop counting and print length in seconds

