%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Construction of device Hamiltonian
%                            from Wannier90 output
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Code constructs Hamiltonian matrices Ham_0 and Ham_T, where Ham_0 is
% interaction within main unit cell, while Ham_T is interaction between main
% unit cell and neighboring unit cell. These matrices are used to describe
% total device Hamiltonian
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%INPUT
%       Wannier90 '_hr' - type file

%OUTPUT 
%       Ham_0 - interaction within the main unit cell
%       Ham_T  - interaction between the main unit cell and neighboring unit cell

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [mat] = NEGF_H_construct(mat)

switch mat.H_input_type

    case "DFT_WANN_1D"  % Optimized for 1D materials

    M = readmatrix(append(num2str(mat.Nw),mat.material_name)+"_hr.dat");
    
    xyz_cell = [max(M(:,1))-min(M(:,1))  max(M(:,2))-min(M(:,2)) max(M(:,3))-min(M(:,3))]+1;
    [xyz_max,xyz_index] = max(xyz_cell);
    
    num_wann = max(M(:,4)); % number of Wannier orbitals in a unit cell
    
    H_all = zeros(num_wann, num_wann, xyz_max); 
    shift = abs(min(M(:,xyz_index)))+1;
    
    %Transformation to H_mni matrix
    for xyz = 1 : xyz_max*num_wann^2
            i = M(xyz,xyz_index) + shift;
    
            m = M(xyz,4);
            n = M(xyz,5);
            
            h_real = M(xyz,6);
            h_imag = M(xyz,7);
            
            h_complex= complex(h_real, h_imag);
            H_all(m,n,i) = real(h_complex); 
    end
    
    NNeighbour = mat.NNeighbour; %Number of nearest neighbour unit cells included in Hamiltonian construction
    
    Ham_0 = kron(diag(ones(1,NNeighbour)),zeros(num_wann, num_wann));
    Ham_T = kron(diag(ones(1,NNeighbour)),zeros(num_wann, num_wann));
    
    for i_a = -NNeighbour+1:NNeighbour-1
        Ham_0 = Ham_0 + kron(diag(ones(1,NNeighbour-abs(i_a)),i_a),H_all(:,:,shift+i_a));
        Ham_T = Ham_T + kron(diag(ones(1,NNeighbour-abs(i_a)),i_a),H_all(:,:,shift+i_a+NNeighbour));
    end

    Ibt = eye(size(Ham_0,1));
    Ham_0 = Ham_0-(mat.Eg_shift)*Ibt;
    
    mat.num_wann = num_wann;
    mat.H_all = H_all;
    mat.shift = shift;
    case 'TB'
        switch mat.material_name
            case 'GNR'
                t1 = 2.7; t2 = 0.2; t3 = 0.18;
                U = 0;
                NU=4; NT=mat.Nl*mat.Nw*NU;

                alphau = [U t1 t2 t3;
                    t1 U t1 t2;
                    t2 t1 U t1;
                    t3 t2 t1 U];

                betaw  = [t2 t1 t2 0;
                    0 t2 t3 0 ;
                    0 t3 t2 0 ;
                    0 t2 t1 t2];
                betal  = [0 0 0 0;
                    0 0 0 0;
                    t2 t3 0 0;
                    t1 t2 0 0];
                betal2 = [0 0 0 0;
                    0 0 0 0;
                    0 0 0 0;
                    t3 t2 0 0];


                Ham_0 = kron(diag(ones(1,mat.Nw)),alphau)+...
                    kron(diag(ones(1,mat.Nw-1),1),betaw)+ ...
                    kron(diag(ones(1,mat.Nw-1),-1),betaw');


                Ham_T = kron(diag(ones(1,mat.Nw)),betal)+...
                    kron(diag(ones(1,mat.Nw-1),1),betal2)+...
                    kron(diag(ones(1,mat.Nw-1),-1),betal2');

                H = sparse( kron(diag(ones(1,mat.Nl)),Ham_0)+kron(diag(ones(1,mat.Nl-1),1),Ham_T)+kron(diag(ones(1,mat.Nl-1),-1),Ham_T') );
        end
    end

mat.Ham_0 = Ham_0;
mat.Ham_T = Ham_T;

end

