%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       TOB calculations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%   Inputs defined in QUDEN_main.m and main_tob.m
%   TOB calculations with:
%        
%   INPUTS:
%        - density of states (DOS) and transmission (T) from NEGF
%        - device parameters, parameters for transfer and output
%          characteristics defined in main_tob.m
%
%   OUTPUTS:
%        - Output and transfer characteristics, quantum capacitance,
%          injection velocity
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function tobiv(t,NWx,file_name_e)

% -------------------------------------------------------------------------
% Inputs imported from NEGF calculations
% -------------------------------------------------------------------------

constants;
load(file_name_e,'mat','d')
E = d.E; dE = d.dE(1); DOS = d.DOS; T = d.T; 
NRWidth = mat.NR_width*1e9; NRLength = mat.NR_length*1e9;

fprintf(1,'\nDevice simulation: length = %f nm, width = %f nm, no. of atoms = %d.\n\n',NRLength, NRWidth, mat.Ntotal);

DOS=DOS/(NRWidth*NRLength*1e-18); 

% -------------------------------------------------------------------------
% DOS and T(TRM0) shift to 0
% remove valence band for NFET, remove conduction band for PFET devices
% -------------------------------------------------------------------------

DOS(DOS<10e5) = 0; DOS(E<-0.03) = 0;
T(T<1e-3)=0; T(E<-0.03) = 0; 

index_min = find(diff(DOS)<0,1,'first');
DOS(1:index_min) = 0; T(1:index_min) = 0;

Emin = E(find(E==E(index_min+1),1,'first'));
Nshift = round(abs(Emin)/dE);
DOS=circshift(DOS,-1*Nshift);
TRM0 = circshift(T,-1*Nshift);

DOS(find(isnan(DOS)==1)) = 1;
TRM0(find(isnan(TRM0)==1)) = 1;

% -------------------------------------------------------------------------
%   Find source fermi level (Efs) from defined doping (frac and Ndmax)
% -------------------------------------------------------------------------

Efs_range=linspace(min(E),max(E),100001); NEfs=length(Efs_range);
Qs_range=zeros(1,NEfs);
Efs = 0;

for k=1:NEfs
    fE_dop=1./(1+exp((E-Efs_range(k))/const.kbT));
    Qs_range(k) = sum(DOS.*fE_dop)*dE; % 1/m2
    if (floor(Qs_range(k)/(t.frac*mat.Ndmax))==(1))
        Efs=Efs_range(k);
        break;
    end
end

% -------------------------------------------------------------------------
% Output and transfer I-V characteristics 
% In main_tob.m - set the OFF state current (Ioff):
%                       'T' - set Ioff, 
%                       'F' - do not set Ioff                   
% -------------------------------------------------------------------------
                                  
Usc0=mat.Eg*0.5 + Efs; % intrinsic channel, Usc0=mat.Eg + 2*Efs channel doped as S/D

if t.I_off_set_switch == 'T'

    for sim_IV = 1 : 2
        if sim_IV == 1
            tobiv_tra; % used to find I_off before setting the I_off to defined value
        elseif sim_IV == 2 
            Vthshift=log(10)*const.kbT*log10(I_off/t.I_off_set); % shift due to I_off_set
            Usc0 = Usc0 + Vthshift; % intrinsic channel % %Usc0=mat.Eg + 2*Efs % channel doped as S/D % Qs0, solve for Vg=0
            tobiv_tra; 
            tobiv_out;
        end
    end

elseif t.I_off_set_switch == 'F'

    sim_IV = 2;
    tobiv_tra; 
    tobiv_out;

end
fprintf(1,'Device simulation DONE\n\n');
% -------------------------------------------------------------------------
% Plot and save TOB data
% -------------------------------------------------------------------------

tobiv_FOMs;
tobiv_save;

end
