function writeFormattedText(filename, names, units, values, formats)
    % writeFormattedText Writes a formatted txt file with names, units, and values
    % Supports custom formats for each column.
    %
    % Parameters:
    %   filename - Name of the output file (string)
    %   names - Cell array of column names
    %   units - Cell array of measurement units
    %   values - Matrix of numerical values
    %   formats - Cell array of formats for each column (e.g., {'%.2f', '%.0f', '%.1f'})
    %
    % Example:
    %   filename = ('qs_vg.txt');
    %   names = {'Vg','Qs'};
    %   units = {'V','1/cm2'};
    %   values = [Vgt(:) 1e-4*Qs_on(:)];
    %   formats = {'%f', '%E'};
    %   writeFormattedTxt(filename, names, units, values, formats);

    % Check input dimensions
    if length(names) ~= length(units)
        error('The number of names and units must be equal.');
    end
    if size(values, 2) ~= length(names)
        error('The number of columns in values must match the number of names and units.');
    end
    if nargin < 5 || isempty(formats)
        % Default to general format if no formats are provided
        formats = repmat({'%g'}, 1, length(names));
    elseif length(formats) ~= length(names)
        error('The number of formats must match the number of names and units.');
    end

    % Open file for writing
    fid = fopen(filename, 'w');
    if fid == -1
        error('Cannot open file: %s', filename);
    end

    % Write names
    fprintf(fid, '%s\t', names{:});
    fprintf(fid, '\n');

    % Write units
    fprintf(fid, '%s\t', units{:});
    fprintf(fid, '\n');

    % Write values row by row with custom formats
    for i = 1:size(values, 1)
        for j = 1:length(names)
            fprintf(fid, formats{j}, values(i, j));
            fprintf(fid, '\t'); % Add a tab after each value
        end
        fprintf(fid, '\n');
    end

    % Close file
    fclose(fid);
    fprintf('File written successfully: %s\n', filename);
end
