%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Material selection
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% Parameters of simulations
% -------------------------------------------------------------------------
% a --> length of a unit cell in transport direction
% b --> width of a unit cell 
% Nw --> number of unit cells in nanoribbon width (used for naming purposes)
% Nw_all_pos --> all possible unit cells (needed for MLWF_DFT inputs)
% Eg_shift --> shift in EK diagram to set 0 at half of the bandgap
% band_number --> number of bands in EK diagram
% CB - > number of lowest conduction band
% VB - > number of highest valence band, calculated as CB - 1
% -------------------------------------------------------------------------

function [mat] = material_selection(mat)

switch mat.material_name
    case 'GNR'
        Nw_all_pos = linspace(1,20,20);
        a0 = -0.142*1e-9; % unit length constant
        a = 0.142*3*1e-9; % unit length
        b0 = -0.071*sqrt(3)*1e-9;
        b = b0 + 0.142*sqrt(3)*1e-9*Nw_all_pos;
        Ncell = 4; % number of atoms in unit cell
        Nadd = 0;  % additional atoms
        CB = Nw_all_pos*2+1;
        Eg = [0.0000000000 ,2.4906853189 ,1.1322604902 ,0.1926086596 ,0.9078906037 ,0.6160292727 ,0.1146850659 ,0.6944953974 ,0.4229781889 ,0.0816491485 ,0.5098813596 ,0.3220324201 ,0.0633889081 ,0.4027645394 ,0.2599801177 ,0.0518033080 ,0.3328270306 ,0.2179760744 ,0.0437982365 ,0.2835787960 ]; 
        Eg_shift = zeros(1,length(Nw_all_pos));
        Eg_shift(mat.Nw) = 0.54013;
        band_number = Nw_all_pos*4;
        mat.H_input_type = "TB";

    case 'HfS2XNR'
        a = 3.62964875*10^-10;
        b = [0,13.60962775,19.97545273,26.25095712,32.53698781,38.826392317,0,51.39819749,0,63.973507214,0,0,0]*10^-10;
        Eg_shift = [0.0000000000 ,-3.0410417958 ,-2.8433934427 ,-2.7150071827 ,-2.6232063240 ,-2.5528057841 ,0.0000000000 ,-2.4588700554 ,0.0000000000 ,-2.3935678595 ];
        Eg = [0.0000000000 ,1.0119454761 ,1.0008983932 ,1.0076835848 ,0.9979979553 ,0.9839042448 ,0.0000000000 ,0.9838726618 ,0.0000000000 ,0.9719015604 ]; 
        band_number = [0,50,72,94,116,138,0,182,0,226];
        CB = [0,31,43,55,67,79,0,103,0,127];
        Ncell = 6;
        Nadd = 2;  
        mat.H_input_type = "DFT_WANN_1D";
    
end

NR_width = b(mat.Nw); % nanoribbon width in m
NR_unit_length = a*mat.NNeighbour; %nanoribbon length of single unit cell in m

mat.a = a*mat.NNeighbour; % length of single unit cell ( unit cell * Number of Neighbours taken into account for single unit cell)
mat.b = b;

mat.Eg_shift = Eg_shift(mat.Nw); % shift in EK diagram to set 0 at half of the bandgap
mat.Eg = Eg(mat.Nw); %bandgap

mat.NR_width = NR_width; % nanoribbon width
mat.NR_unit_length = NR_unit_length; % nanoribbon length of single unit cell in m
mat.NR_unit_area = NR_unit_length*NR_width; % nanoribbon area of single unit cell in m2
mat.NU = band_number(mat.Nw)*mat.NNeighbour; % number of bands/Wannier orbitals in a unitcell
mat.N_unit = (Ncell*mat.Nw + Nadd)*mat.NNeighbour; %total number of atoms in single unit cell 

mat.CB = CB; % lowest conduction band
mat.VB = CB - 1; % highest valence band 

if mat.H_input_type == 'TB'

    mat.Nl = round((mat.Lx_c-a0)/mat.a); % set number of unit cells in device length (channel)
    mat.NR_length = mat.NR_unit_length*mat.Nl+a0; % total nanoribbon length (channel)
    mat.NR_area  = mat.NR_length*mat.NR_width;   % total nanoribbon area (channel)

else % if mat.H_input_type == DFT_WANN_1D

    mat.Nl = round(mat.Lx_c/mat.a); % set number of unit cells in device length (channel)
    mat.NR_length = mat.NR_unit_length*mat.Nl; % total nanoribbon length (channel)
    mat.NR_area  = mat.NR_unit_area*mat.Nl;   % total nanoribbon area (channel)

end

mat.Ntotal = mat.N_unit * mat.Nl; % total number of atoms in device
mat.Ndmax = mat.Ntotal/mat.NR_area; % areal atomic density (1/m2)

end
